import {Component, ViewChild}         from '@angular/core';
import {ElementRef, Input}            from '@angular/core';
import {ActivatedRoute, Router}       from '@angular/router';
import {Title, Meta}                  from '@angular/platform-browser';

import {Observable}                   from 'rxjs/Observable';

import {DatasetInfo}                  from '../../utils/entities/datasetInfo';
import {EnvProperties}                from '../../utils/properties/env-properties';
import {RouterHelper}                 from '../../utils/routerHelper.class';

import {DatasetService}               from './dataset.service';
import {PiwikService}                 from '../../utils/piwik/piwik.service';
import { SEOService } from '../../sharedComponents/SEO/SEO.service';


@Component({
    selector: 'dataset',
    templateUrl: 'dataset.component.html',
})

export class DatasetComponent {
    @Input() piwikSiteId = null;
    public datasetInfo: DatasetInfo;
    public datasetId : string ;

    // APP BOX variables
    public showAllCollectedFrom: boolean = false;
    public showAllDownloadFrom: boolean = false;
    public showAllPublishedIn: boolean = false;

    // Metrics tab variables
    public metricsClicked: boolean;
    public viewsFrameUrl: string;
    public downloadsFrameUrl: string;
    public totalViews: number;
    public totalDownloads: number;
    public pageViews: number;

    // Active tab variable for responsiveness
    public activeTab: string = "Related Research Results";

    // Map counting variable
    public relatedResearchResultsNum: number = 0;

    // Message variables
    public warningMessage = "";
    public errorMessage = "";
    public showLoading: boolean = true;

    public routerHelper:RouterHelper = new RouterHelper();

    private result ;
    sub: any; piwiksub: any; infoSub: any;
    properties:EnvProperties;

    constructor (private element: ElementRef,
                 private _datasetService: DatasetService,
                 private _piwikService:PiwikService,
                 private  route: ActivatedRoute,
                 private _meta: Meta,
                 private _title: Title,
                 private _router: Router,
               private seoService: SEOService) {
    }

    ngOnInit() {
      this.route.data
        .subscribe((data: { envSpecific: EnvProperties }) => {
           this.properties = data.envSpecific;
           this.updateUrl(data.envSpecific.baseLink+this._router.url);

        });
      this.sub =  this.route.queryParams.subscribe(params => {
          this.datasetInfo = null;
          this.updateTitle("Dataset");
          this.updateDescription("Dataset, search, open access");
          this.seoService.createLinkForCanonicalURL();

            this.datasetId = params['datasetId'];
             console.info("Id is :"+this.datasetId);

             if(this.datasetId){
                this.getDatasetInfo(this.datasetId);
            }else{
              this.showLoading = false;
                this.warningMessage="No valid research data id";
            }

            this.metricsClicked = false;

            this.viewsFrameUrl = this.properties.framesAPIURL+'merge.php?com=query&data=[{"query":"resRepoViews", "resTitle":"'+this.datasetId+'", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":30,"sort":"xaxis","xStyle":{"r":-30,"s":"0","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly views"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true&persistent=false';
            /*this.viewsFrameUrl = this.properties.framesAPIURL+'merge.php?com=query&data=[{"query":"resViewsTimeline", "resTitle":"'+this.datasetId+'", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":30,"sort":"xaxis","xStyle":{"r":-30,"s":"0","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly views"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(124, 181, 236, 1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true&persistent=false';
            */

            this.downloadsFrameUrl = this.properties.framesAPIURL+'merge.php?com=query&data=[{"query":"resRepoDownloads", "resTitle":"'+this.datasetId+'", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":30,"sort":"xaxis","xStyle":{"r":-30,"s":"0","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly downloads"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true&persistent=false';
            /*this.downloadsFrameUrl = this.properties.framesAPIURL+'merge.php?com=query&data=[{"query":"resRepoDownloadTimeline", "resTitle":"'+this.datasetId+'", "table":"","fields":[{"fld":"sum","agg":"sum","type":"column","yaxis":1,"c":false}],"xaxis":{"name":"month","agg":"sum"},"group":" ","color":"","type":"chart","size":30,"sort":"xaxis","xStyle":{"r":-30,"s":"0","l":"-","ft":"-","wt":"-"},"title":"","subtitle":"","xaxistitle":"Repository","yaxisheaders":["Monthly downloads"],"generalxaxis":"","theme":0,"in":[],"filters":[{"name":"","values":[""],"to":"-1"}]}]&info_types=["column"]&stacking=normal&steps=false&fontFamily=Courier&spacing=[5,0,0,0]&style=[{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"rgba(0, 0, 0, 1)","size":"18"},{"color":"000000","size":""},{"color":"000000","size":""}]&backgroundColor=rgba(255,255,255,1)&colors[]=rgba(67, 67, 72, 1)&colors[]=rgba(144, 237, 125, 1)&colors[]=rgba(247, 163, 92, 1)&colors[]=rgba(128, 133, 233, 1)&colors[]=rgba(241, 92, 128, 1)&colors[]=rgba(228, 211, 84, 1)&colors[]=rgba(43, 144, 143, 1)&colors[]=rgba(244, 91, 91, 1)&colors[]=rgba(145, 232, 225, 1)&xlinew=0&ylinew=1&legends=true&tooltips=true';
            */
            if (typeof document !== 'undefined') {
               this.element.nativeElement.scrollIntoView();
            }
        });

    }

    ngOnDestroy() {
      if(this.sub){
        this.sub.unsubscribe();
      }
      if(this.piwiksub){
        this.piwiksub.unsubscribe();
      }
      if(this.infoSub) {
        this.infoSub.unsubscribe();
      }
    }

    private getDatasetInfo(id:string) {
        this.warningMessage = '';
        this.errorMessage=""
        this.showLoading = true;

        this.datasetInfo = null;

        this.infoSub = this._datasetService.getDatasetInfo(id, this. properties).subscribe(
            data => {
                this.datasetInfo = data;
                /*
                if(this.datasetInfo.title.name != undefined && this.datasetInfo.title.name!= ''){
                  this.updateTitle(this.datasetInfo.title.name);
                  this.updateDescription("Dataset, search, repositories, open access,"+this.datasetInfo.title.name);
                }
                */
                if(this.datasetInfo.title){
                  this.updateTitle(this.datasetInfo.title);
                  this.updateDescription("Dataset, search, repositories, open access,"+this.datasetInfo.title);
                }
                if(this.properties.enablePiwikTrack && (typeof document !== 'undefined')){
                  this.piwiksub = this._piwikService.trackView(this.properties, this.datasetInfo.title/*.name*/, this.piwikSiteId).subscribe();
                }

                this.result = []
                this.result = {id: id, type :"dataset", source : "openaire", title: this.datasetInfo.title,url: '', result: '', accessRights: /*this.datasetInfo.title.accessMode*/ this.datasetInfo.accessMode, embargoEndDate: ''};

                let relatedResearchResultsNum = 0;
                if(this.datasetInfo.relatedResearchResults != undefined) {
                    this.datasetInfo.relatedResearchResults.forEach(function (value, key, map) {
                        relatedResearchResultsNum += value.length;
                    });
                }
                this.relatedResearchResultsNum = relatedResearchResultsNum;

                this.showLoading = false;
            },
            err => {
                console.log(err)
                console.info("error");

                this.errorMessage = 'No research data found';
                this.showLoading = false;
            }
        );
    }

    // showChange($event) {
    //     this.showAllReferences=$event.value;
    // }

    public metricsResults($event) {
      this.totalViews = $event.totalViews;
      this.totalDownloads = $event.totalDownloads;
      this.pageViews = $event.pageViews;
    }

    private updateDescription(description:string){
      this._meta.updateTag({content:description},"name='description'");
      this._meta.updateTag({content:description},"property='og:description'");
    }
    private updateTitle(title:string){
      var _prefix ="OpenAIRE | ";
      var _title = _prefix + ((title.length> 50 ) ?title.substring(0,50):title);
      this._title.setTitle(_title);
      this._meta.updateTag({content:_title},"property='og:title'");
    }
    private updateUrl(url:string){
      this._meta.updateTag({content:url},"property='og:url'");
    }

    public buildCurationTooltip() {
        let tooltipContent: string = "<div class='uk-padding-small uk-light'>";

        tooltipContent += "<h4>Record in preview</h4>";
        tooltipContent += "<p>Bibliographic record accepted by the system, but not yet processed by <br> OpenAIRE tools for information quality improvement and de-duplication</p></di>";

        return tooltipContent;
    }
    public getKeys( map) {
      return Array.from(map.keys());
    }
}
