import {Component, Input, ViewChild} from '@angular/core';
import { ActivatedRoute} from '@angular/router';
import { Filter, Value} from '../searchUtils/searchHelperClasses.class';
import {SearchDataprovidersService} from '../../services/searchDataproviders.service';
import {SearchResult}     from '../../utils/entities/searchResult';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {SearchFields, FieldDetails} from '../../utils/properties/searchFields';
import {SearchPageTableViewComponent } from '../searchUtils/searchPageTableView.component';
import {SearchUtilsClass } from '../searchUtils/searchUtils.class';
import{EnvProperties} from '../../utils/properties/env-properties';

@Component({
    selector: 'search-entity-registries-table',
    template: `

    <search-page-table pageTitle="Registries/ Databases"
                       type="content providers" entityType="dataprovider" [(filters)] = "filters"
                       [(results)] = "results" [(searchUtils)] = "searchUtils"
                       [columnNames]="columnNames"
                       [showResultCount]=false
                       [disableForms]="disableForms"
                       [enableSearchView]="enableSearchView"
                       [searchViewLink]="'/search/entity-registries'"
                       searchFormClass="entityRegistriesTableSearchForm"
                       formPlaceholderText="Search for Registries/ Databases">
    </search-page-table>

    `

})
export class SearchEntityRegistriesTableComponent {
  private errorCodes: ErrorCodes;

  public results =[];
  public filters =[];
  public columnNames = ["Name", "Type", "Country", "Institution", "Compatibility"];
  public baseUrl:string;
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  public sub: any; public subResults: any;
  public _location:Location;
  public searchFields:SearchFields = new SearchFields();
  public refineFields: string[] =  this.searchFields.ENTITY_REGISTRIES_FIELDS;
  public disableForms: boolean = false;
  public enableSearchView: boolean = true;
properties:EnvProperties;
  @ViewChild (SearchPageTableViewComponent) searchPage : SearchPageTableViewComponent ;

  constructor (private route: ActivatedRoute, private _searchDataprovidersService: SearchDataprovidersService ) {
    this.errorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;
  }

  public ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;
        this.baseUrl = data.envSpecific.searchLinkToEntityRegistriesDataProviders;

      });
    this.searchPage.refineFields = this.refineFields;
    this.sub =  this.route.queryParams.subscribe(params => {
      this.searchUtils.keyword = (params['keyword']?params['keyword']:'');
      //this.searchUtils.page = (params['page']=== undefined)?1:+params['page'];
      this.filters = this.createFilters();

      this.searchPage.getParametersFromUrl(params);
      this._getResults("", false, this.searchUtils.page);
    });
  }

  public ngOnDestroy() {
    if(this.sub){
      this.sub.unsubscribe();
    }
    if(this.subResults){
      this.subResults.unsubscribe();
    }  }
    private _getResults(parameters:string,refine:boolean, page: number){
      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;
      this.disableForms = true;
      this.enableSearchView = false;
      this.results = [];
      this.searchUtils.totalResults = 0;

      let size: number = 0;
      this.subResults = this._searchDataprovidersService.searchEntityRegistriesTable(this.properties).subscribe(
          data => {
              size = data;
              if(size > 0) {
                this.subResults = this._searchDataprovidersService.searchEntityRegistries("",null, page, size, [],this.properties).subscribe(
                    data => {
                        this.searchUtils.totalResults = data[0];
                        console.info("search Entity Registries [total results:"+this.searchUtils.totalResults+"]");
                        this.results = data[1];
                        this.searchPage.checkSelectedFilters(this.filters);

                        //var errorCodes:ErrorCodes = new ErrorCodes();
                        this.searchUtils.status = this.errorCodes.DONE;
                        if(this.searchUtils.totalResults == 0 ){
                          this.searchUtils.status = this.errorCodes.NONE;
                        } else {
                          this.searchPage.triggerInitialLoad();
                          this.searchPage.transform(this.results);
                          this.disableForms = false;
                        }
                        this.enableSearchView = true;
                    },
                    err => {
                        console.log(err);
                         //TODO check erros (service not available, bad request)
                        // if( ){
                        //   this.searchUtils.status = ErrorCodes.ERROR;
                        // }
                        //var errorCodes:ErrorCodes = new ErrorCodes();
                        //this.searchUtils.status = errorCodes.ERROR;
                        if(err.status == '404') {
                          this.searchUtils.status = this.errorCodes.NOT_FOUND;
                        } else if(err.status == '500') {
                          this.searchUtils.status = this.errorCodes.ERROR;
                        } else {
                          this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
                        }
                        this.enableSearchView = true;
                    }
                );
              } else {
                this.searchPage.checkSelectedFilters(this.filters);

                //var errorCodes:ErrorCodes = new ErrorCodes();
                this.searchUtils.status = this.errorCodes.NONE;
                this.enableSearchView = true;
              }
          },
          err => {
              console.log(err);
               //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = ErrorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.ERROR;
              if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }
              this.enableSearchView = true;
          }
      );
/*
       this.subResults = this._searchDataprovidersService.searchEntityRegistries(parameters,(refine)?this.searchPage.getRefineFieldsQuery():null, page, size, []).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              console.info("search Entity Registries: [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
              this.results = data[1];
              this.searchPage.checkSelectedFilters(this.filters);
              this.searchPage.updateBaseUrlWithParameters(this.filters);
              var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = errorCodes.NONE;
              }
              //this.searchPage.closeLoading();
              this.disableForms = false;

          },
          err => {
              console.log(err);
               //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = ErrorCodes.ERROR;
              // }
              var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = errorCodes.ERROR;
              //this.searchPage.closeLoading();
              this.disableForms = false;

          }
      );
*/
    }
  private setFilters(){
    //TODO set filters from
  }


  private createFilters():Filter[] {
    var filter_names=["Type","Compatibility"];
    var filter_ids=["datasourcetypename","datasourcecompatibilityname"];
    var searchFields = new SearchFields();
    var filter_original_ids = searchFields.ENTITY_REGISTRIES_FIELDS;
    var value_names=[
      ["Funder database","Registry of repositories","Scholarly Comm. Infrastructure","Registry","Information Space","Web Source"],

      ["OpenAIRE Basic (DRIVER OA)","OpenAIRE 2.0 (EC funding)", "OpenAIRE 2.0+ (DRIVER OA, EC funding)", "OpenAIRE 3.0 (OA, funding)","OpenAIRE Data (funded, referenced datasets)",
       "collected from a compatible aggregator", "proprietary", "under validation"]];

    var value_original_ids=[
      ["Funder database","Registry of repositories","Scholarly Comm. Infrastructure","Registry","Information Space","Web Source"],
      //["entityregistry::projects","entityregistry::repositories","scholarcomminfra","entityregistry","infospace","websource"],
      //["driver","openaire2.0", "driver-openaire2.0", "openaire3.0","openaire2.0_data"]
    ["OpenAIRE Basic (DRIVER OA)","OpenAIRE 2.0 (EC funding)", "OpenAIRE 2.0+ (DRIVER OA, EC funding)", "OpenAIRE 3.0 (OA, funding)","OpenAIRE Data (funded, referenced datasets)",
     "collected from a compatible aggregator", "proprietary", "under validation"]];

    var filters: Filter[] =[];
    for(var i =0 ; i < filter_names.length;i++){
      var values:Value[] = [];
      for(var j =0 ; j < value_names[i].length;j++){
        var value:Value = {name: value_names[i][j], id: value_original_ids[i][j], number:j, selected:false}
        values.push(value);
      }
       var filter:Filter = {title: filter_names[i], filterId: filter_ids[i], originalFilterId:  filter_original_ids[i], values : values, countSelectedValues:0, "filterOperator": 'or' };
       filters.push(filter);
    }
    return filters;
    }
}
